from da.ctdata import fluxload

#
# collections are defined below as a pick from stateparameters
# the name of the stateparameter should be identical to the flux it scales in TM5
#

co2_bioNHgirdded = {
    "variance": 0.64,
    "cov_t_length": 0,
    "cyclic": False,
    "water": False,
    "mean": 1.0,
    "fluxname": "sib4v2462022",
    "regions": "regions",
    "griddedtc" : [1,2,7,8,11],
    "gridded_length_scales" : [300,300,1000,1000,300],
    "within_tcregions": [3, 4, 5, 6, 9, 10],
    "length_scales": [1000]*6,
    "coupled" : ['co2c13_bio'],
}
co2_ocnNHgridded = {
    "variance": 0.16,
    "cyclic": False,
    "water": True,
    "mean": 1.0,
    "fluxname": "carboscopev2022",
    "regions": "regions",
    "length_scales": [1000],
    "coupled" : ['co2c13_ocn'],
}

#
#  Scales gpp, known to ctdas as sib4gpp
co2_bio = {
    "variance": 0.64,
    "cov_t_length": 3,
    "cyclic": False,
    "water": False,
    "freq": "monthly",
    "mean": 1.0,
    "fluxname": "sib4isobio",
    "regions": "regions",
    "within_tcregions": [3, 4, 5, 6, 9, 10],
    "coupled" : ['co2c13_bio'],
}



co2c13_recor = {
    "variance": 2.5e-5,
    "cov_t_length": 4,
    "cyclic": False,
    "water": False,
    "freq": "monthly",
    "mean": 0.0,
    "func": fluxload.rrecoload,
    "regions": "regions",
    "within_tcregions": [3, 4, 5, 6, 9, 10],
}

co2c13_alpha = {
    "variance": 2.5e-5,
    "cov_t_length": 3,
    "cyclic": False,
    "water": False,
    "freq": "monthly",
    "mean": 0.0,
    "func": fluxload.alphagppload,
    "regions": "regions",
    # "within_tcregions": [3, 4, 5, 6, 9, 10],
}

co2_gppiav = {
    "variance": 0.25,
    "cov_t_length": 4,
    "cyclic": False,
    "water": False,
    "freq": "daily",
    "mean": 1.0,
    "fluxname": "sib4247gppiav",
    "regions": "regions",
    "within_tcregions": [3, 4, 5, 6, 9, 10],
    "coupled" : ['co2c13_gppiav'],
    "limit_region_unc" : True,
    "limit_factor" : 4,
}
co2_recoclim = {
    "variance": 0.16,
    "cov_t_length": 35,
    "cyclic": True,
    "water": False,
    "freq" : "daily",
    "mean" : 1.0,
    "fluxname": "sib4247recoclim",
    "regions": "regions",
    "within_tcregions": [3, 4, 5, 6, 9, 10],
    "coupled" : ['co2c13_recoclim'],
    "limit_region_unc" : True,
    "limit_factor" : 0.1,
}

co2_gpp = {
    "variance": 0.16,
    "cov_t_length": 4,
    "cyclic": False,
    "water": False,
    "freq": "daily",
    "mean": 1.0,
    "fluxname": "sib4gpp",
    "regions": "regions",
    "within_tcregions": [3, 4, 5, 6, 9, 10],
    "coupled" : ['co2c13_gpp'],
    "limit_region_unc" : True,
    "limit_factor" : 0.1,
}

co2_ocn = {
    "variance": 0.2,
    "cov_t_length": 3,
    "cyclic": False,
    "water": True,
    "freq": "monthly",
    "mean": 1.0,
    "fluxname": "carboscopev2022",
    "regions": "regions",
    "coupled" : ['co2c13_ocn'],
}

co2c13_ocndis = {
    "variance": 0.02,
    "cov_t_length": 3,
    "cyclic": False,
    "water": True,
    "freq": "monthly",
    "mean": 1.0,
    "fluxname": "sib4ocndis",
    "regions": "transcom_regions",
    # "length_scales": [3000],
}
co2c13_biodis = {
    "variance": 0.02,
    "cov_t_length": 3,
    "cyclic": False,
    "water": False,
    "freq": "monthly",
    "mean": 1.0,
    "fluxname": "sib4biodis",
    "regions": "transcom_regions",
    # "length_scales": [3000],
}
NHgridded = {"co2_bio" : co2_bioNHgirdded, "co2_ocn" : co2_ocnNHgridded}

LW_TER =    {"co2_recoclim": co2_recoclim, "co2_ocn": co2_ocn}

LW_TERGPP = {"co2_gppiav": co2_gppiav, "co2_recoclim": co2_recoclim, "co2_ocn": co2_ocn}

LW_TERGPPv2 = {"co2_gpp": co2_gpp, "co2_recoclim": co2_recoclim, "co2_ocn": co2_ocn}
LW_C13_veldeno = {
    "co2_bio": co2_bio,
    "co2c13_biodis": co2c13_biodis,
    "co2_ocn": co2_ocn,
    "co2c13_ocndis": co2c13_ocndis,
    }
LW_C13_velde = {
    "co2_bio": co2_bio,
    "co2c13_biodis": co2c13_biodis,
    "co2_ocn": co2_ocn,
    "co2c13_ocndis": co2c13_ocndis,
    "co2c13_gppalpha": co2c13_alpha,
    }
LW_C13 = {
    "co2_gpp" : co2_gpp,
    "co2_recoclim": co2_recoclim,
    "co2_ocn": co2_ocn,
    "co2c13_ocndis": co2c13_ocndis,
    "co2c13_gppalpha": co2c13_alpha,
    "co2c13_recor": co2c13_recor,
}

statevector_collection = {
    "LW_C13_velde": LW_C13_velde,
    "LW_C13_veldeno": LW_C13_veldeno,
    "LW_C13": LW_C13,
    "NHgridded": NHgridded,
    "LW_TER": LW_TER,
    "LW_TERGPP": LW_TERGPP,
    "LW_TERGPPv2": LW_TERGPPv2,
}
